import click


@click.command()
@click.argument('branch_name')
@click.argument('commit_name')
@click.argument('commit_description')
@click.option('--pull_request', '-pr', is_flag=True, default=False)
def apply(branch_name, commit_name, commit_description, pull_request):
    """Pull, create branch, add, commit, push and make an optional PR"""
    from octohot.cli.config import repositories
    for repo in repositories:
        if repo.tainted():
            click.echo(repo.print("Apply (Pull, change branch, add files, "
                       "commit, push and make a optional PR)"))
            repo.pull()
            repo.branch(branch_name)
            repo.add()
            repo.commit(commit_name, commit_description)
            repo.push(branch_name)
            if pull_request and repo.is_github():
                from octohot.cli.github.pr import _pr
                _pr(repo, commit_name, commit_description, branch_name)
            repo.branch(repo.default_branch())




import click

from octohot.cli.config import config

try:
    ORG = config['github_organization']
except:
    ORG = None

@click.group()
@click.option('--name', '-n', default=None)
def org(name=None):
    """Organization from GitHub provider"""
    global ORG

    if name:
        ORG = name
    else:
        if not ORG:
            raise Exception("Missing organization")


from octohot.cli.github.organization.org_import import organization_import
org.add_command(organization_import)

from octohot.cli.github.organization.list import list
org.add_command(list)


def url_list(url_type):
    from github3 import GitHub
    from octohot.cli.github.github import GITHUB_TOKEN
    github = GitHub()
    github.login(token=GITHUB_TOKEN)
    org = github.organization(ORG)
    repos = org.repositories()
    if url_type == 'https':
        urls = [repo.clone_url for repo in repos]
    else:
        urls = [repo.ssh_url for repo in repos]
    return urls




import click


@click.group()
def regex():
    """RegEx provider for octohot"""


from octohot.cli.regex.find import find
regex.add_command(find)

from octohot.cli.regex.replace import replace
regex.add_command(replace)

if __name__ == '__main__':
    regex()



import click


@click.command()
def sync():
    """Sync: Clone, reset, delete unpushed Branches, pull"""
    from octohot.cli.config import repositories
    for repo in repositories:
        click.echo(repo.print("Sync: Clone, reset, delete unpushed Branches, "
                              "pull"))
        repo.clone()
        repo.reset()
        repo.branch(repo.default_branch())
        repo.delete_unpushed_branches()
        repo.pull()



import click


@click.command()
@click.argument('commit_name')
@click.argument('commit_description')
def commit(commit_name, commit_description):
    """Commit added changes in all repos"""
    from octohot.cli.config import repositories
    for repo in repositories:
        if repo.tainted():
            click.echo(repo.print(f'Commit {commit_name}'))
            repo.commit(commit_name, commit_description)



import os
from octohot.sh import sh


def path(exists=True, raise_error=True):
    """
        Decorator: Execution depends if folder exists or not
    :param exists: Boolean. Check if exists or not.
    :param raise_error: Boolean. Check if there will be an error if exists check
    :return:
    """
    def decorator(fn):
        def wrapper(*args, **kwargs):
            self = args[0]
            if os.path.isdir(self.folder) == exists:
                return fn(*args, **kwargs)
            else:
                if raise_error:
                    if exists:
                        FileNotFoundError(self.folder)
                    else:
                        FileExistsError(self.folder)
        return wrapper
    return decorator


class Repository():

    def __init__(self, url):
        self.url = url
        self.name = url.split('/')[-1].replace('.git', '')
        self.folder = f'{os.getcwd()}/{self.name}'

        self.owner = self.url.split('/')[-2]
        if ':' in self.owner:
            self.owner = self.owner.split(':')[-1]

    def print(self, msg):
        cmd = f'>>>>>> {self.name}: {msg}'
        return cmd

    def clone(self):
        cmd = 'git clone --single-branch %s %s'
        return sh(cmd % (self.url, self.folder))

    def pull(self):
        cmd = 'git -C %s pull' # TODO: --merge
        return sh(cmd % self.folder)

    def reset(self):
        """
            Reseta a branch atual, baseada na respectiva branch remota.
        """
        # 'git -C %s clean -fd ' % self.folder
        # 'git -C %s checkout -- .' % self.folder
        cmd = 'git -C %s reset --hard origin/$(git -C %s rev-parse ' \
              '--abbrev-ref HEAD)'
        return sh(cmd % (self.folder, self.folder))

    def diff(self):
        cmd = 'git -C %s --no-pager diff'
        return sh(cmd % self.folder)

    def branch(self, branch_name):
        cmd = 'git -C %s checkout %s 2>/dev/null || ' \
              'git -C %s checkout master; git -C %s checkout -b %s'
        cmd = cmd % (self.folder, branch_name, self.folder, self.folder,
                     branch_name)
        sh(cmd)

    def add(self):
        sh('git -C %s add .' % self.folder)

    def commit(self, commit_name, commit_description):
        commit_name = commit_name.replace('"', '\\"')
        commit_description = commit_description.replace('"', '\\"')
        sh('git -C %s commit -m "%s" -m "%s"'
           % (self.folder, commit_name, commit_description))

    def push(self, branch_name):
        sh('git -C %s push origin %s' % (self.folder, branch_name))

    def default_branch(self):
        cmd = """git -C %s symbolic-ref refs/remotes/origin/HEAD | \
            sed 's@^refs/remotes/origin/@@'"""
        return sh(cmd % self.folder)[0]

    def all_branches(self):
        cmd = """git -C %s branch | cut -c 3-"""
        return sh(cmd % self.folder)

    def delete_branch(self, branch_name):
        cmd = """git -C %s branch -D %s"""
        return sh(cmd % (self.folder, branch_name))

    def delete_unpushed_branches(self):
        branches_to_remove = [branch for branch in self.all_branches()
                              if not branch == self.default_branch()]
        return [self.delete_branch(branch) for branch in branches_to_remove]

    def tainted(self):
        """
        Lista arquivos que foram incluídos, modificados ou preparados (staged)
        :return:
        """
        files = sh(
            'git -C %s status -uall -s' % self.folder
        )
        return bool(len(files))

    def is_current_branch(self, branch):
        current_branch = sh(
            """git -C %s branch | grep \* | cut -d' ' -f2""" % self.folder
        )[0]
        return current_branch == branch

    def is_github(self):
        return 'github.com' in self.url

    def branch_has_diff_from_master(self, branch_name):
        diff = sh(f"git -C {self.folder} --no-pager diff master..{branch_name}")

        # branch not exists
        if 'unknown revision or path' in '\n'.join(diff):
            return False

        return bool(len(diff))




import click
import logging

from octohot.cli.regex.find import _find
from octohot.sh import sh


def _replace(repo, files, find_str, replace_str):
    """
        using regex groups:
            value = "1teste2"
            old = "[0-9](.*)[0-9]"
            new = "W\1Q"
            result = "WtesteQ"
    :param files:
    :param find_str:
    :param replace_str:
    :return:
    """
    find_str = find_str.replace('|', '\|').replace('"', '\\"')
    replace_str = replace_str.replace('|', '\|').replace('"', '\\"')
    find_str = find_str.replace("'", "\\x27")
    cmd = """sed -Ei "s|%s|%s|g" %s"""
    [sh(cmd % (find_str, replace_str, arq)) for arq in files]

    info = "Replaced '%s' to '%s' in repo %s:\n%s"
    info = info % (find_str, replace_str, repo.name, '\n'.join(files))
    logging.info(info)
    print(info)


@click.command()
@click.argument('find_string')
@click.argument('replace_string')
def replace(find_string, replace_string):
    """Find and replace a string in all repos"""
    from octohot.cli.config import repositories
    for repo in repositories:
        files = _find(repo, find_string)
        if len(files):
            _replace(repo, files, find_string, replace_string)
            click.echo('Replacing files:'+'\n'.join(files))


import click


@click.command(name='import')
@click.argument('url_type', type=click.Choice(['https', 'ssh']), default='ssh')
def organization_import(url_type):
    """Import all repositories to octohot.yml config file from a Organization
    from GitHub"""
    from octohot.cli.github.organization.organization import url_list
    urls = url_list(url_type)

    from octohot.cli.config import config, save
    missed = [url for url in urls if url not in config['repositories']]
    [config['repositories'].append(url) for url in missed]
    save()
    click.echo("Added %s new repositories in config file" % len(missed))




import click
import logging
from octohot.sh import sh


def _find(repo, find_str):
    # find_str = find_str.replace('|', '\|').replace('"', '\\"')
    find_str = find_str.replace("'", "\\x27")
    cmd = "grep -PRl '%s' %s --exclude-dir=.git"
    files = sh(cmd % (find_str, repo.folder))
    if files:
        logging.info(
            '\'%s\' found in repo %s: \n'
            '%s' % (find_str, repo.name, '\n'.join(files))
        )
        return files
    return []


def matches(files, find_str):
    all_matches = []
    for file in files:
        cmd = "grep -Pohn '%s' %s --exclude-dir=.git"
        all_matches += sh(cmd % (find_str, file))

        logging.info(
            'Patterns founded in file %s : \n'
            '%s' % (file, '\n'.join(files))
        )
    return all_matches


@click.command()
@click.argument('find_string')
@click.option('--only_filepath', '-p', is_flag=True,
              help='print only filepaths')
def find(find_string, only_filepath):
    """Find a regular expression in all repos"""
    from octohot.cli.config import repositories
    for repo in repositories:
        files = _find(repo, find_string)
        if files:
            click.echo('\n'.join(files))
            if not only_filepath:
                click.echo('\n'.join(matches(files, find_string)))



import click


@click.command()
def diff():
    """Get diff from all repos"""
    from octohot.cli.config import repositories
    for repo in repositories:
        if repo.tainted():
            click.echo(repo.print("Diff"))
            print('\n'.join(repo.diff()))



import click


@click.command(name='list')
@click.option('--type', '-t', type=click.Choice(['https', 'ssh']),
              default='https')
def list(type):
    """List all repositories to octohot.yml config file from a Organization
    from GitHub"""
    from octohot.cli.github.organization.organization import url_list
    print('\n'.join(url_list(type)))



import click


@click.group()
def git():
    """git provider for octohot"""


from octohot.cli.git.clone import clone
git.add_command(clone)

from octohot.cli.git.add import add
git.add_command(add)


from octohot.cli.git.pull import pull
git.add_command(pull)

from octohot.cli.git.reset import reset
git.add_command(reset)

from octohot.cli.git.branch import branch
git.add_command(branch)

from octohot.cli.git.commit import commit
git.add_command(commit)

from octohot.cli.git.push import push
git.add_command(push)

from octohot.cli.git.diff import diff
git.add_command(diff)

if __name__ == '__main__':
    git()



import click

@click.group()
@click.option('--verbose', '-v', is_flag=True, help='Enables verbose mode.')
def cli(verbose=True):
    """Command line interface for the octohot package"""
    if verbose:
        import logging
        logging.getLogger().setLevel(logging.INFO)


@click.command()
def version():
    """Display the current version."""
    import pkg_resources  # part of setuptools
    version = pkg_resources.require("octohot")[0].version
    click.echo(f'version: {version}')


cli.add_command(version)

from octohot.cli.sync import sync
cli.add_command(sync)

# from octohot.cli.regex.find import find
# cli.add_command(find)

# from octohot.cli.regex.replace import replace
# cli.add_command(replace)

from octohot.cli.apply import apply
cli.add_command(apply)

from .git import git
cli.add_command(git.git)

from .github import github
cli.add_command(github.github)

from .regex import regex
cli.add_command(regex.regex)

if __name__ == '__main__':
    cli()



import click


@click.command()
def pull():
    """Pull all repos"""
    from octohot.cli.config import repositories
    for repo in repositories:
        click.echo(repo.print("Pull"))
        repo.pull()



default_filename = 'octohot.yml'
default_config = {
    'repositories': [],
    'github_token': None,
    'github_organization': None

}

repositories = config = None

import os
config_file = os.path.join(os.getcwd(), default_filename)


def load():
    global config
    global repositories

    import os.path
    if os.path.isfile(config_file):
        import yaml
        config = yaml.load(open(config_file, "r"))
    else:
        config = default_config

    from octohot.cli.git.repository import Repository
    repositories = [Repository(repo_name) for repo_name in
                    config['repositories']]


def save():
    import yaml
    yaml.dump(config, open(config_file, "w"), default_flow_style=False)


load()



import click


@click.command()
@click.argument('pr_title')
@click.argument('pr_comment')
@click.argument('branch_name')
def pr(pr_title, pr_comment, branch_name):
    from octohot.cli.config import repositories
    for repo in repositories:
        if repo.is_github() and repo.is_current_branch(branch_name):
            _pr(repo, pr_title, pr_comment, branch_name)


def _pr(repo, pr_title, pr_comment, branch_name):
    from github3 import GitHub
    from octohot.cli.github.github import GITHUB_TOKEN
    github = GitHub()
    github.login(token=GITHUB_TOKEN)
    github_repo = github.repository(repo.owner, repo.name)
    github_repo.create_pull(pr_title, 'master', branch_name, pr_comment)
    click.echo(f"PR github repo {repo.name} PR {pr_title} branch {branch_name}")



import click


@click.command()
def add():
    """Add all unstaged files to stage"""
    from octohot.cli.config import repositories
    for repo in repositories:
        if repo.tainted():
            click.echo(repo.print("Add"))
            repo.add()



import click


@click.command()
@click.argument('branch_name')
def push(branch_name):
    """Push all repos"""
    from octohot.cli.config import repositories
    for repo in repositories:
        if repo.branch_has_diff_from_master(branch_name):
            click.echo(repo.print(f"Push {branch_name}"))
            repo.push(branch_name)



import click


@click.command()
def clone():
    """Clone all repos"""
    from octohot.cli.config import repositories
    for repo in repositories:
        click.echo(repo.print("Clone"))
        repo.clone()



import click


@click.command()
@click.argument('branch_name')
def branch(branch_name):
    """Create/Change branch in all repos"""
    from octohot.cli.config import repositories
    for repo in repositories:
        click.echo(repo.print(f'Branch {branch_name}'))
        repo.branch(branch_name)



import click

from octohot.cli.config import config

try:
    GITHUB_TOKEN = config['github_token']
except:
    GITHUB_TOKEN = None

@click.group()
@click.option('--token', '-t')
def github(token=None):
    """GitHub provider for octohot"""
    global GITHUB_TOKEN
    if token:
        GITHUB_TOKEN = token
    else:
        if not GITHUB_TOKEN:
            raise Exception("Missing token")


from octohot.cli.github.pr import pr
github.add_command(pr)

from octohot.cli.github.organization.organization import org
github.add_command(org)

